@@ -2,58 +2,59 @@ use crate::prelude::*;
 
 /// General function that can create any entry type.
 ///
-/// This is used under the hood by [ `create_entry` ], [ `create_cap_grant` ] and [ `create_cap_claim` ].
+/// This is used under the hood by [`create_entry`], [`create_cap_grant`] and [`create_cap_claim`].
 ///
-/// The host builds a [ `Create` ] header for the passed entry value and commits a new element to the
+/// The host builds a [`Create`] header for the passed entry value and commits a new element to the
 /// chain.
 ///
 /// Usually you don't need to use this function directly; it is the most general way to create an
-/// entry and standardises the internals of higher level create functions.
+/// entry and standardizes the internals of higher level create functions.
 pub fn create(create_input: CreateInput) -> ExternResult<HeaderHash> {
     HDK.with(|h| h.borrow().create(create_input))
 }
 
-/// Update any entry type.
+/// General function that can update any entry type.
+///
+/// This is used under the hood by [`update_entry`], [`update_cap_grant`] and `update_cap_claim`.
 ///
-/// This is used under the hood by [ `update_entry` ], [ `update_cap_grant` ] and `update_cap_claim`.
 /// @todo implement update_cap_claim
 ///
-/// The host builds an [ `Update` ] header for the passed entry value and commits a new update to the
+/// The host builds an [`Update`] header for the passed entry value and commits a new update to the
 /// chain.
 ///
 /// Usually you don't need to use this function directly; it is the most general way to update an
-/// entry and standardises the internals of higher level create functions.
+/// entry and standardizes the internals of higher level update functions.
 pub fn update(hash: HeaderHash, create_input: CreateInput) -> ExternResult<HeaderHash> {
     HDK.with(|h| h.borrow().update(UpdateInput::new(hash, create_input)))
 }
 
 /// General function that can delete any entry type.
 ///
-/// This is used under the hood by [ `delete_entry` ], [ `delete_cap_grant` ] and `delete_cap_claim`.
+/// This is used under the hood by [`delete_entry`], [`delete_cap_grant`] and `delete_cap_claim`.
+///
 /// @todo implement delete_cap_claim
 ///
-/// The host builds a [ `Delete` ] header for the passed entry and commits a new element to the chain.
+/// The host builds a [`Delete`] header for the passed entry and commits a new element to the chain.
 ///
-/// Usually you don't need to use this function directly; it is the most general way to update an
-/// entry and standardises the internals of higher level create functions.
-pub fn delete(deletes_header_address: HeaderHash) -> ExternResult<HeaderHash> {
-    HDK.with(|h| {
-        h.borrow().delete(DeleteInput::new(
-            deletes_header_address,
-            ChainTopOrdering::default(),
-        ))
-    })
+/// Usually you don't need to use this function directly; it is the most general way to delete an
+/// entry and standardizes the internals of higher level delete functions.
+pub fn delete<I, E>(delete_input: I) -> ExternResult<HeaderHash>
+where
+    DeleteInput: TryFrom<I, Error = E>,
+    WasmError: From<E>,
+{
+    HDK.with(|h| h.borrow().delete(DeleteInput::try_from(delete_input)?))
 }
 
-/// Create an app entry.
+/// Create an app entry. Also see [`create`].
 ///
 /// Apps define app entries by registering entry def ids with the `entry_defs` callback and serialize the
 /// entry content when committing to the source chain.
 ///
-/// This function accepts any input that implements [ `TryInto<CreateInput>` ].
-/// The default impls from the `#[hdk_entry( .. )]` and [ `entry_def!` ] macros include this.
+/// This function accepts any input that implements [`TryInto<CreateInput>`].
+/// The default impls from the `#[hdk_entry( .. )]` and [`entry_def!`] macros include this.
 ///
-/// With generic type handling it may make sense to directly construct [ `CreateInput` ] and [ `create` ].
+/// With generic type handling it may make sense to directly construct [`CreateInput`] and [`create`].
 ///
 /// e.g.
 /// ```ignore
@@ -62,7 +63,7 @@ pub fn delete(deletes_header_address: HeaderHash) -> ExternResult<HeaderHash> {
 /// create_entry(Foo(50))?;
 /// ```
 ///
-/// See [ `get` ] and [ `get_details` ] for more information on CRUD.
+/// See [`get`] and [`get_details`] for more information on CRUD.
 pub fn create_entry<I, E>(input: I) -> ExternResult<HeaderHash>
 where
     CreateInput: TryFrom<I, Error = E>,
@@ -71,69 +72,37 @@ where
     create(CreateInput::try_from(input)?)
 }
 
-/// Alias to delete
+/// Delete an app entry. Also see [`delete`].
 ///
-/// Takes the [ `HeaderHash` ] of the element to delete.
+/// This function accepts the [`HeaderHash`] of the element to delete and optionally an argument to
+/// specify the [`ChainTopOrdering`]. Refer to [`DeleteInput`] for details.
 ///
 /// ```ignore
-/// delete_entry(entry_hash(foo_entry)?)?;
+/// #[hdk_entry(id = "foo")]
+/// struct Foo(u32);
+///
+/// let header_hash = create_entry(Foo(50))?;
+/// let delete_entry_header_hash = delete_entry(header_hash.clone())?;
 /// ```
-pub fn delete_entry(hash: HeaderHash) -> ExternResult<HeaderHash> {
-    delete(hash)
-}
-
-/// Hash anything that that implements [ `TryInto<Entry>` ].
-///
-/// Hashes are typed in holochain, e.g. [ `HeaderHash` ] and [ `EntryHash` ] are different and yield different
-/// bytes for a given value. This ensures correctness and allows type based dispatch in various
-/// areas of the codebase.
-///
-/// Usually you want to hash a value that you want to reference on the DHT with [ `get` ] etc. because
-/// it represents some domain-specific data sourced externally or generated within the wasm.
-/// [ `HeaderHash` ] hashes are _always_ generated by the process of committing something to a local
-/// chain. Every host function that commits an entry returns the new [ `HeaderHash` ]. The [ `HeaderHash` ] can
-/// also be used with [ `get` ] etc. to retreive a _specific_ element from the DHT rather than the
-/// oldest live element.
-/// However there is no way to _generate_ a header hash directly from a header from inside wasm.
-/// [ `Element` ] values (entry+header pairs returned by [ `get` ] etc.) contain prehashed header structs
-/// called [ `HeaderHashed` ], which is composed of a [ `HeaderHash` ] alongside the "raw" [ `Header` ] value. Generally the pre-hashing is
-/// more efficient than hashing headers ad-hoc as hashing always needs to be done at the database
-/// layer, so we want to re-use that as much as possible.
-/// The header hash can be extracted from the Element as `element.header_hashed().as_hash()`.
-/// @todo is there any use-case that can't be satisfied by the `header_hashed` approach?
-///
-/// Anything that is annotated with #[hdk_entry( .. )] or entry_def!( .. ) implements this so is
-/// compatible automatically.
-///
-/// [ `hash_entry` ] is "dumb" in that it doesn't check that the entry is defined, committed, on the DHT or
-/// any other validation, it simply generates the hash for the serialized representation of
-/// something in the same way that the DHT would.
-///
-/// It is strongly recommended that you use the [ `hash_entry` ] function to calculate hashes to avoid
-/// inconsistencies between hashes in the wasm guest and the host.
-/// For example, a lot of the crypto crates in rust compile to wasm so in theory could generate the
-/// hash in the guest, but there is the potential that the serialization logic could be slightly
-/// different, etc.
 ///
+/// with a specific [`ChainTopOrdering`]:
 /// ```ignore
-/// #[hdk_entry(id="foo")]
-/// struct Foo;
-///
-/// let foo_hash = hash_entry(Foo)?;
+/// delete_entry(DeleteInput::new(header_hash.clone(), ChainTopOrdering::Relaxed)?;
 /// ```
-pub fn hash_entry<I, E>(input: I) -> ExternResult<EntryHash>
+pub fn delete_entry<I, E>(delete_input: I) -> ExternResult<HeaderHash>
 where
-    Entry: TryFrom<I, Error = E>,
+    DeleteInput: TryFrom<I, Error = E>,
     WasmError: From<E>,
 {
-    HDK.with(|h| h.borrow().hash_entry(Entry::try_from(input)?))
+    delete(delete_input)
 }
 
-/// Thin wrapper around update for app entries.
-/// The hash is the [ `HeaderHash` ] of the deleted element, the input is a [ `TryInto<CreateInput>` ].
+/// Update an app entry. Also see [`update`].
+///
+/// The hash is the [`HeaderHash`] of the deleted element, the input is a [`TryInto<CreateInput>`].
 ///
 /// Updates can reference Elements which contain Entry data -- namely, Creates and other Updates -- but
-/// not Deletes or system Elements
+/// not Deletes or system Elements.
 ///
 /// As updates can reference elements on other agent's source chains across unpredictable network
 /// topologies, they are treated as a tree structure.
@@ -154,9 +123,9 @@ where
 /// @todo in the future this will be true because we will have the concept of 'redirects':
 /// Works as an app entry delete+create.
 ///
-/// See [ `create_entry` ]
-/// See [ `update` ]
-/// See [ `delete_entry` ]
+/// See [`create_entry`]
+/// See [`update`]
+/// See [`delete_entry`]
 pub fn update_entry<I, E>(hash: HeaderHash, input: I) -> ExternResult<HeaderHash>
 where
     CreateInput: TryFrom<I, Error = E>,
@@ -173,15 +142,15 @@ where
 ///
 /// An element is no longer live once it is referenced by a valid delete element.
 /// An update to an element does not change its liveness.
-/// See [ `get_details` ] for more information about how CRUD elements reference each other.
+/// See [`get_details`] for more information about how CRUD elements reference each other.
 ///
-/// Note: [ `get` ] __always triggers and blocks on a network call__.
+/// Note: [`get`] __always triggers and blocks on a network call__.
 ///       @todo implement a 'get optimistic' that returns based on the current opinion of the world
 ///       and performs network calls in the background so they are available 'next time'.
 ///
 /// Note: Deletes are considered in the liveness but Updates are not currently followed
 ///       automatically due to the need for the happ to disambiguate update logic.
-///       @todo implement 'redirect' logic so that updates are followed by [ `get` ].
+///       @todo implement 'redirect' logic so that updates are followed by [`get`].
 ///
 /// Note: Updates typically point to a different entry hash than what they are updating but not
 ///       always, e.g. consider changing `foo` to `bar` back to `foo`. The entry hashes in a crud
@@ -191,7 +160,7 @@ where
 ///
 /// Note: "oldest live" only relates to disambiguating many creates and updates from many authors
 ///       pointing to a single entry, it is not the "current value" of an entry in a CRUD sense.
-///       e.g. If "foo" is created then updated to "bar", a [ `get` ] on the hash of "foo" will return
+///       e.g. If "foo" is created then updated to "bar", a [`get`] on the hash of "foo" will return
 ///            "foo" as part of an element with the "oldest live" header.
 ///            To discover "bar" the agent needs to call `get_details` and decide how it wants to
 ///            collapse many potential creates, updates and deletes down into a single or filtered
@@ -281,7 +250,7 @@ pub fn must_get_header(header_hash: HeaderHash) -> ExternResult<SignedHeaderHash
     })
 }
 
-/// MUST get a VALID Element at a given HeaderHash
+/// MUST get a VALID Element at a given HeaderHash.
 ///
 /// The Element is guaranteed to be valid.
 /// More accurately the Element is guarantee to be consistently reported as valid by the visible network.
@@ -327,9 +296,9 @@ pub fn must_get_valid_element(header_hash: HeaderHash) -> ExternResult<Element>
     })
 }
 
-/// Get an element from the hash AND the details for the entry or header hash passed in.
-/// Returns [ `None` ] if the entry/header does not exist.
-/// The details returned are a contextual mix of elements and header hashes, see below.
+/// Get an element and its details for the entry or header hash passed in.
+/// Returns [`None`] if the entry/header does not exist.
+/// The details returned are a contextual mix of elements and header hashes.
 ///
 /// Note: The return details will be inferred by the hash type passed in, be careful to pass in the
 ///       correct hash type for the details you want.
@@ -337,7 +306,7 @@ pub fn must_get_valid_element(header_hash: HeaderHash) -> ExternResult<Element>
 /// Note: If a header hash is passed in the element returned is the specified element.
 ///       If an entry hash is passed in all the headers (so implicitly all the elements) are
 ///       returned for the entry that matches that hash.
-///       See [ `get` ] for more information about what "oldest live" means.
+///       See [`get`] for more information about what "oldest live" means.
 ///
 /// The details returned include relevant creates, updates and deletes for the hash passed in.
 ///
@@ -350,11 +319,11 @@ pub fn must_get_valid_element(header_hash: HeaderHash) -> ExternResult<Element>
 /// Full elements are returned for direct references to the passed hash.
 /// Header hashes are returned for references to references to the passed hash.
 ///
-/// [ `Details` ] for a header hash return:
+/// [`Details`] for a header hash return:
 /// - the element for this header hash if it exists
 /// - all update and delete _elements_ that reference that specified header
 ///
-/// [ `Details` ] for an entry hash return:
+/// [`Details`] for an entry hash return:
 /// - all creates, updates and delete _elements_ that reference that entry hash
 /// - all update and delete _elements_ that reference the elements that reference the entry hash
 ///
@@ -367,10 +336,10 @@ pub fn must_get_valid_element(header_hash: HeaderHash) -> ExternResult<Element>
 ///
 /// Note: There are multiple header types that exist and operate entirely outside of CRUD elements
 ///       so they cannot reference or be referenced by CRUD, so are immutable or have their own
-///       mutation logic (e.g. link create/delete) and will not be included in [ `get_details` ] results
+///       mutation logic (e.g. link create/delete) and will not be included in [`get_details`] results
 ///       e.g. the DNA itself, links, migrations, etc.
-///       However the element will still be returned by [ `get_details` ] if a header hash is passed,
-///       these header-only elements will have [ `None` ] as the entry value.
+///       However the element will still be returned by [`get_details`] if a header hash is passed,
+///       these header-only elements will have [`None`] as the entry value.
 pub fn get_details<H: Into<AnyDhtHash>>(
     hash: H,
     options: GetOptions,
@@ -385,8 +354,8 @@ pub fn get_details<H: Into<AnyDhtHash>>(
         .unwrap())
 }
 
-/// Trait for binding static [ `EntryDef` ] property access for a type.
-/// See [ `register_entry` ]
+/// Trait for binding static [`EntryDef`] property access for a type.
+/// See [`register_entry`]
 pub trait EntryDefRegistration {
     fn entry_def() -> crate::prelude::EntryDef;
 
@@ -403,8 +372,8 @@ pub trait EntryDefRegistration {
 /// If you have some need to implement custom serialization logic or metadata injection
 /// you can do so by implementing these traits manually instead.
 ///
-/// This requires that TryFrom and TryInto [ `derive@SerializedBytes` ] is implemented for the entry type,
-/// which implies that [ `serde::Serialize` ] and [ `serde::Deserialize` ] is also implemented.
+/// This requires that TryFrom and TryInto [`derive@SerializedBytes`] is implemented for the entry type,
+/// which implies that [`serde::Serialize`] and [`serde::Deserialize`] is also implemented.
 /// These can all be derived and there is an attribute macro that both does the default defines.
 #[macro_export]
 macro_rules! app_entry {
@@ -447,7 +416,7 @@ macro_rules! app_entry {
             fn try_from(element: &$crate::prelude::Element) -> Result<Self, Self::Error> {
                 Ok(match element.entry() {
                     ElementEntry::Present(entry) => Self::try_from(entry)?,
-                    _ => return Err(Self::Error::Guest(format!("Tried to deserialize an element, expecting it to contain entry data, but there was none. Element HeaderHash: {}", element.header_hashed().as_hash()))),
+                    _ => return Err(Self::Error::Guest(format!("Tried to deserialize an element, expecting it to contain entry data, but there was none. Element HeaderHash: {}", element.header_address()))),
                 })
             }
         }
@@ -502,7 +471,7 @@ macro_rules! app_entry {
 /// This allows crates to easily define a struct as an entry separately to binding that struct
 /// as an entry type in a dependent crate.
 ///
-/// For most normal applications, you should use the [ `entry_def!` ] macro instead.
+/// For most normal applications, you should use the [`entry_def!`] macro instead.
 #[macro_export]
 macro_rules! register_entry {
     ( $t:ident $def:expr ) => {
@@ -657,10 +626,10 @@ macro_rules! register_entry {
 /// impls are just a loose set of conventions.
 ///
 /// It's actually entirely possible to interact with core directly without any of these.
-/// e.g. [ `create_entry` ] is just building a tuple of [ `EntryDefId` ] and [ `Entry::App` ] under the hood.
+/// e.g. [`create_entry`] is just building a tuple of [`EntryDefId`] and [`Entry::App`] under the hood.
 ///
-/// This requires that TryFrom and TryInto [ `derive@SerializedBytes` ] is implemented for the entry type,
-/// which implies that [ `serde::Serialize` ] and [ `serde::Deserialize` ] is also implemented.
+/// This requires that TryFrom and TryInto [`derive@SerializedBytes`] is implemented for the entry type,
+/// which implies that [`serde::Serialize`] and [`serde::Deserialize`] is also implemented.
 /// These can all be derived and there is an attribute macro that both does the default defines.
 ///
 ///  e.g. the following are equivalent
@@ -711,18 +680,18 @@ macro_rules! entry_defs {
     };
 }
 
-/// Attempts to lookup the [ `EntryDefIndex` ] given an [ `EntryDefId` ].
+/// Attempts to lookup the [`EntryDefIndex`] given an [`EntryDefId`].
 ///
-/// The [ `EntryDefId` ] is a [ `String` ] newtype and the [ `EntryDefIndex` ] is a u8 newtype.
-/// The [ `EntryDefIndex` ] is used to reference the entry type in headers on the DHT and as the index of the type exported to tooling.
-/// The [ `EntryDefId` ] is the 'human friendly' string that the [ `entry_defs!` ] callback maps to the index.
+/// The [`EntryDefId`] is a [`String`] newtype and the [`EntryDefIndex`] is a u8 newtype.
+/// The [`EntryDefIndex`] is used to reference the entry type in headers on the DHT and as the index of the type exported to tooling.
+/// The [`EntryDefId`] is the 'human friendly' string that the [`entry_defs!`] callback maps to the index.
 ///
 /// The host actually has no idea how to do this mapping, it is provided by the wasm!
 ///
-/// Therefore this is a macro that calls the [ `entry_defs!` ] callback as defined within a zome directly from the zome.
+/// Therefore this is a macro that calls the [`entry_defs!`] callback as defined within a zome directly from the zome.
 /// It is a macro so that we can call a function with a known name `crate::entry_defs` from the HDK before the function is defined.
 ///
-/// Obviously this assumes and requires that a compliant [ `entry_defs!` ] callback _is_ defined at the root of the crate.
+/// Obviously this assumes and requires that a compliant [`entry_defs!`] callback _is_ defined at the root of the crate.
 #[macro_export]
 macro_rules! entry_def_index {
     ( $t:ty ) => {
